using System;
using System.Drawing;

namespace dynablaster.client.gui.gameplay
{
	/// <summary>
	/// helper klasse fr die errechnung der bildschirmkoordinaten 
	/// der zu zeichnenden spielobjekte und spielitems
	/// </summary>
	public class FieldLayoutCalculator
	{
		//sizes of game stuff (players, fields, bombs)
		private const int FIELD_WIDTH=35;
		private const int FIELD_HEIGHT=35;
		private const int WIDTH_PLAYER=15;
		private const int HEIGHT_PLAYER=15;
        private const int WIDTH_BOMB=7;
		private const int HEIGHT_BOMB=7;

		/// <summary>
		/// x-Koordinate, wo das Spielfeld anfngt
		/// </summary>
		private const int START_X=170;
		/// <summary>
		/// y-Koordinate, wo das Spielfeld anfng
		/// </summary>
		private const int START_Y=40;

		//


		/// <summary>
		/// bersetzt ein x/y-Koordinaten des Spielfeldes in einen
		/// Bildpunkt, damit ein "richtiges" Zeichnen des Spielobjektes mglich ist
		/// </summary>
		/// <param name="x">Die nummer der Spalte, erste Spalte ist links mit
		/// wert 0</param>
		/// <param name="y">Die Nummer der Spalte, erste Reihe ist oben mit Wert
		/// 0</param>
		/// <returns>Grafikpunkt zum Zeichnen</returns>
		private static Point CalcObjPosFromCoord(int x, int y)
		{
			int pointX, pointY;
			//ausrechnen horizontal
			pointX= START_X+(x*FieldLayoutCalculator.FIELD_WIDTH);
			//ausrechnen vertikal
			pointY= START_Y+(y*FieldLayoutCalculator.FIELD_HEIGHT); 
			return new Point(pointX,pointY);
		}

		
		/// <summary>
		/// bersetzt ein x/y-Koordinaten des Spielfeldes in einen
		/// Bildpunkt, damit ein "richtiges" Zeichnen des Items Spieler
		/// Dabei sitzt das Item im Zentrum eines Spielobjektes
		/// </summary>
		/// <param name="x">Nummer der Spalte, erste Spalte ist links mit wert
		/// 0</param>
		/// <param name="y">Nummer der Zeile, erste Reihe ist oben mit 
		/// Wert 0</param>
		/// <returns>Grafikpunkt zum Zeichnen</returns>
		private static Point CalcPlayerPosFromCoord(int x, int y)
		{
			int pointX,pointY;
			//X coordinate bekommen, zunchst spaltenpunkt bekommen
			pointX=START_X+(x*FieldLayoutCalculator.FIELD_WIDTH);
			//jetzt position des item im Spielobjekt selber
			pointX=(int)pointX+FIELD_WIDTH/2-WIDTH_PLAYER/2;
			//Y coordinate bekommen, zunchst reihenpunkt bekommen
			pointY=START_Y+(y*FieldLayoutCalculator.FIELD_HEIGHT); 
			//jetzt position des item im Spielobjekt selber
			pointY=(int)pointY+FIELD_HEIGHT/2-HEIGHT_PLAYER/2;
			return new Point(pointX,pointY);
		}

		/// <summary>
		/// bersetzt ein x/y-Koordinaten des Spielfeldes in einen
		/// Bildpunkt, damit ein "richtiges" Zeichnen des Items Bombe.
		/// Dabei sitzt das Item im Zentrum eines Spielobjektes
		/// </summary>
		/// <param name="x">Nummer der Spalte, erste Spalte ist links mit wert
		/// 0</param>
		/// <param name="y">Nummer der Zeile, erste Reihe ist oben mit 
		/// Wert 0</param>
		/// <returns>Grafikpunkt zum Zeichnen</returns>
		private static Point CalcBombPosFromCoord(int x, int y)
		{
			int pointX,pointY;
			//X coordinate bekommen, zunchst spaltenpunkt bekommen
			pointX=START_X+(x*FieldLayoutCalculator.FIELD_WIDTH);
			//jetzt position des item im Spielobjekt selber
			pointX=(int)pointX+FIELD_WIDTH/2-WIDTH_BOMB/2;
			//Y coordinate bekommen, zunchst reihenpunkt bekommen
			pointY=START_Y+(y*FieldLayoutCalculator.FIELD_HEIGHT); 
			//jetzt position des item im Spielobjekt selber
			pointY=(int)pointY+FIELD_HEIGHT/2-HEIGHT_BOMB/2;
			return new Point(pointX,pointY);
		}


		
		
		/// <summary>
		/// Erstellt ein Rechteck eines Spielobjektes, welches einen Bereich 
		/// des Fensters zeigt, so dass einfach dieses von Graphics gezeichnet 
		/// bzw. gefllt werden kann. 
		/// </summary>
		/// <param name="x">Die x-Koordinate des Spielfeldes</param>
		/// <param name="y">Die y-Koordinate des Spielfeldes</param>
		/// <returns>Rechteck, welches von Graphics passend verwendet werden kann</returns>
		public static Rectangle GetObjectRectFromPoint(int x, int y)
		{
			Size size= new Size(FieldLayoutCalculator.FIELD_WIDTH,FieldLayoutCalculator.FIELD_HEIGHT);
			Point p=FieldLayoutCalculator.CalcObjPosFromCoord(x,y);
			return new Rectangle(p,size);
		}


		/// <summary>
		/// Erstellt ein Rechteck fr einen Spieler an gegebener Koordinate
		/// </summary>
		/// <param name="x">Die x-Koordinate des Spielfeldes</param>
		/// <param name="y">Die y-Koordinate des Spielfeldes</param>
		/// <returns>Rechteck fr Spielitem</returns>
		public static Rectangle GetPlayerRectFromPoint(int x, int y)
		{
			Size size=new Size(FieldLayoutCalculator.WIDTH_PLAYER,FieldLayoutCalculator.HEIGHT_PLAYER);
			Point p=FieldLayoutCalculator.CalcPlayerPosFromCoord(x,y);
			return new Rectangle(p,size);

		}

		/// <summary>
		/// Erstellt ein Rechteck fr eine Bombe an gegebener Koordinate
		/// </summary>
		/// <param name="x">Die x-Koordinate des Spielfeldes</param>
		/// <param name="y">Die y-Koordinate des Spielfeldes</param>
		/// <returns>Rechteck fr Spielitem</returns>
		public static Rectangle GetBombRectFromPoint(int x, int y)
		{
			Size size=new Size(FieldLayoutCalculator.WIDTH_BOMB,FieldLayoutCalculator.HEIGHT_BOMB);
			Point p=FieldLayoutCalculator.CalcBombPosFromCoord(x,y);
			return new Rectangle(p,size);
		}

	}
}
